/*=================================================================*/
/*
 * Parallel Recursive Spectral Bisection -
 * Based on the sequential version provided by Horst Simon
 * -ravi
 */
/*=================================================================*/
#include  <math.h>
#include <stdio.h>
#include "parti.h"

/*=================================================================*/
/* utility functions called by the lanczos solver                */
/* givens() */
/* scal() */
/* nrm2() */
/* pivot() */
/* fudge() */
/* copy () */
/* dot() */
/* axpy () */
/* swap() */
/* -ravi 06/23/91 */
/*=================================================================*/

/* givens */
void
givens(a, b, n, eig, evec)
double *a, *b;
int n;
double eig, *evec;
{

  int i;
  double sum, bd; 

  evec[n] = 1.0;
  evec[n-1] = (eig -a[n])*evec[n]/b[n];
  sum = 1.0 + evec[n-1]*evec[n-1];
  for(i=n-2; i>=0; i--) {
    evec[i] = ((eig - a[i+1]) * evec[i+1] - b[i+2] * evec[i+2])/ b[i+1];
    sum = sum + evec[i]*evec[i];
  }
  bd = 1.0 / sqrt(sum);
  for(i=0; i<=n; i++) {
    evec[i] = evec[i] * bd;
  }
  return;
}
/*=================================================================*/

void
givens_old(a, b, n, eig, evec)
double *a, *b;
int n;
double eig, *evec;
{

  int i;
    double sum, bd;

   evec[n-1] = 1.0;
   evec[n-2] = (eig -a[n-1])*evec[n-1]/b[n-1];
   sum = 1.0 + evec[n-2]*evec[n-2];
   for(i=n-3; i>=0; i--) {
      evec[i] = ((eig - a[i+1]) * evec[i+1] - b[i+2] * evec[i+2])/ b[i+1];
      sum = sum + evec[i]*evec[i];
   }
   bd = 1.0 / sqrt(sum);
   for(i=0; i<n; i++) {
      evec[i] = evec[i] * bd;
   }
   return;
}
/*=================================================================*/


/* scal */
void scal(n, a, x)
int n;
double a, *x;
{
  int i;
  for(i=0; i<n; i++) {
#ifdef DEBUG
    printf("IN SCAL U %f * %f  = %f\n", x[i], a , a*x[i]); 
#endif
    x[i] = a*x[i];
  }
  return;
}
/*=================================================================*/

/* nrm2 */
double
nrm2(n, x)
double  *x;
int n;
{
   int i;
   double sum;
   double work;

   sum = 0.0;
   for(i=0; i<n; i++) {
     sum += (x[i]*x[i]);
   }
   PARTI_gdsum(&sum, 1, &work); 
 
   sum = (double)sqrt((double)sum);
/*
   printf("IN NRM2 RES %f \n", sum);
*/
   return sum;
}
/*=================================================================*/

/* pivot */
void
pivot(a, b, n, x, dout)
double *a, *b, x, *dout;
int n;
{
   int i;
   double d;

   d = 1.0;
   if (n <= 0) { 
     *dout = d; 
     return;
   }
   d = a[0] - x;
   if (d == 0.0) d = b[0]*1e-6;
   if (n == 1) { 
     *dout = d; 
     return;
   }
   for (i=1; i<n; i++) {
      d = a[i]-x-b[i-1]/d;
      if (d == 0.0) d = b[i-1]*1e-6;
   }
   *dout = d;
  return;
}
/*=================================================================*/
/* fudge */
void 
fudge(evec,  neqns)
int neqns;
double *evec;
{
    int i;

    for(i=0; i<neqns; i++) {
/*
       evec[i] = (double) (2*mask[i]) + evec[i];
*/
       evec[i] = (double)2 + evec[i];
    }
    return;
}
/*=================================================================*/

/* copy */
void
copy(n, x, y)
int n;
double *x, *y;
{
   int i;

   for(i=0; i<n; i++) {
      y[i] = x[i];
   }
   return;
}
/*=================================================================*/

/* dot */
double 
dot(n, x, y)
double *x, *y;
int n;
{
   int i;
   double sum;
   double work;

   sum = 0.0;
   for(i=0; i<n; i++) {
      sum += x[i]*y[i];
   }
   
/*
   printf("IN DOT sum %f\n", sum);
*/
   PARTI_gdsum(&sum, 1, &work);
   return sum;
}
/*=================================================================*/

/* axpy */
void 
axpy(n, a, x, y)
double a, *x, *y;
int n;
{

   int i;

   for(i=0; i<n; i++) {
     y[i] = y[i] + a*x[i];
   }
   return;
}
/*=================================================================*/

/* swap */
void
swap(n, x, y)
double *x, *y;
int n;
{
   int i;
   double temp;
   
   for(i=0; i<n; i++) {
     temp = y[i];
     y[i] = x[i];
     x[i] = temp;
   }
   return;
}
/*=================================================================*/
